/* pam_data.c */

#include <stdlib.h>
#include <string.h>

#include "pam_appl.h"
#include "pam_private.h"

struct pam_data *_locate_data(pam_handle_t *pamh, const char *name, int add);

int pam_set_data(
    pam_handle_t *pamh,
    const char *module_data_name,
    const void *data,
    int (*cleanup)(pam_handle_t *pamh, void *data, int error_status))
{
    struct pam_data *data_entry;

    data_entry = _locate_data(pamh, module_data_name, 1);

    /* XXX - Should cleanup() really be called here? */
    if (data_entry->cleanup) {
	data_entry->cleanup(pamh, data_entry->data, PAM_SUCCESS);
    }

    if (data_entry->name) {
	free(data_entry->name);
    }

    data_entry->name = strdup(module_data_name);
    data_entry->data = data;
    data_entry->cleanup = cleanup;

    return PAM_SUCCESS;
}

int pam_get_data(
    const pam_handle_t *pamh,
    const char *module_data_name,
    void **datap)
{
    struct pam_data *data;

    data = _locate_data(pamh, module_data_name, 0);
    if (data) {
	*datap = data->data;
	return PAM_SUCCESS;
    }

    return PAM_NO_MODULE_DATA;
}

struct pam_data *_locate_data(pam_handle_t *pamh, const char *name, int add)
{
    struct pam_data *data = pamh->data;
    struct pam_data *new;
    
    while (data) {
	if (!strcmp(data->name, name)) {
	    return data;
	}
	data = data->next;
    }

    if (add) {
	new = malloc(sizeof(*new));
	new->name = NULL;
	new->data = NULL;
	new->cleanup = NULL;
	new->next = pamh->data;
	pamh->data = new;
	return new;
    }

    return NULL;
}

void _pam_free_data (pam_handle_t *pamh, int status)
{
    struct pam_data *data = pamh->data;
    struct pam_data *last;

    while (data) {
	last = data;
	data = data->next;
	if (last->cleanup) {
	    last->cleanup(pamh, last->data, status);
	}
	free(last->name);
	free(last);
    }
}
