/* pam_item.c */

#include <stdlib.h>
#include <string.h>

#include "pam_appl.h"
#include "pam_private.h"

#define RESET(X, Y) \
if (X) { \
     free(X); \
}; \
(X) = (Y) ? strdup(Y) : NULL

int pam_set_item (
    pam_handle_t *pamh,
    int item_type,
    const void *item)
{
    switch (item_type) {
    case PAM_SERVICE:
	_pam_free_handlers(pamh);
	RESET(pamh->service_name, item);
	/* XXX - need to handle errors here */
	_pam_init_handlers(pamh);
	break;
    case PAM_USER:
	RESET(pamh->user, item);
	break;
    case PAM_TTY:
	RESET(pamh->tty, item);
	break;
    case PAM_RUSER:
	RESET(pamh->ruser, item);
	break;
    case PAM_RHOST:
	RESET(pamh->rhost, item);
	break;
    case PAM_AUTHTOK:
	RESET(pamh->authtok, item);
	break;
    case PAM_OLDAUTHTOK:
	RESET(pamh->oldauthtok, item);
	break;
    case PAM_CONV:
	pamh->pam_conversation = item;
        break;
    default:
	return PAM_BAD_ITEM;
    }

    return PAM_SUCCESS;
}

int pam_get_item (
    const pam_handle_t *pamh,
    int item_type,
    void **item)
{
    switch (item_type) {
    case PAM_SERVICE:
	*item = pamh->service_name;
	break;
    case PAM_USER:
	*item = pamh->user;
	break;
    case PAM_TTY:
	*item = pamh->tty;
	break;
    case PAM_RUSER:
	*item = pamh->ruser;
	break;
    case PAM_RHOST:
	*item = pamh->rhost;
	break;
    case PAM_AUTHTOK:
	*item = pamh->authtok;
	break;
    case PAM_OLDAUTHTOK:
	*item = pamh->oldauthtok;
	break;
    case PAM_CONV:
	*item = pamh->pam_conversation;
	break;
    default:
	return PAM_BAD_ITEM;
    }
  
    return PAM_SUCCESS;
}
